'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _loremIpsum = require('lorem-ipsum');

var _loremIpsum2 = _interopRequireDefault(_loremIpsum);

var _cliColor = require('cli-color');

var _cliColor2 = _interopRequireDefault(_cliColor);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _ajaxRequest = require('ajax-request');

var _ajaxRequest2 = _interopRequireDefault(_ajaxRequest);

var _es6Promise = require('es6-promise');

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Utils = function () {
  function Utils() {
    _classCallCheck(this, Utils);

    this._form = {};
    this._key = [];
  }

  /**
   * Get all input from a template
   * @return {Array} array of input form
   */


  _createClass(Utils, [{
    key: 'dontHaveKey',


    /**
     * Check if key is not is the form array
     * @param  {[type]} key [description]
     * @return {[type]}     [description]
     */
    value: function dontHaveKey(key) {
      return typeof this._key[key] === "undefined" || this._key[key] === null;
    }

    /**
     * Add entry to abe engine form
     * @param {String} type        textarea | text | meta | link | image | ...
     * @param {String} key         unique ID, no space allowed
     * @param {String} desc        input description
     * @param {Int}    maxLength   maximum characteres allowed inside input
     * @param {String} tab         tab name
     * @param {String} jsonValue   
     * @return {Void}
     */

  }, {
    key: 'add',
    value: function add(type, key, desc, maxLength, tab, jsonValue, source, display, reload, order, required) {
      var value = '';
      var block = '';
      if (typeof jsonValue !== 'undefined' && jsonValue !== null) {
        value = jsonValue;
      }

      if (key.indexOf('[') < 0 && key.indexOf('.') > -1) {
        block = key.split('.')[0];
      }

      if (tab === '' || typeof tab === 'undefined' || tab === null) tab = 'default';
      if (typeof this._form[tab] === 'undefined' || this._form[tab] === null) this._form[tab] = { item: [] };

      this._key[key] = true; // save key for dontHaveKey()
      this._form[tab].item.push({
        type: type,
        key: key.replace(/\./, '-'),
        description: desc,
        maxLength: maxLength,
        placeholder: '',
        value: value,
        tab: tab,
        source: source,
        order: order,
        required: required,
        reload: typeof reload !== 'undefined' && reload !== null && reload === 'true' ? true : false,
        display: display,
        block: block
      });
    }
  }, {
    key: 'isBlock',
    value: function isBlock(str) {
      return str.indexOf('[') < 0 && str.indexOf('.') > 0;
    }

    /**
     * Test if a string don't contains string key from ABE block statement
     * @param  {String}  str string to test
     * @return {Boolean} true = this is not a block content
     */

  }, {
    key: 'isSingleAbe',
    value: function isSingleAbe(str, text) {
      return !new RegExp('#each(.)+?' + (0, _.getAttr)(str, 'key').split('.')[0]).test(text) && str.indexOf('{{#') < 0 && str.indexOf('#each') < 0 && str.indexOf('{{/') < 0 && str.indexOf('/each') < 0 && str.indexOf('attrAbe') < 0;
    }

    /**
     * Test if a string contains string key from ABE block statement
     * @param  {String}  str string to test
     * @return {Boolean} true = this is a block content
     */

  }, {
    key: 'isBlockAbe',
    value: function isBlockAbe(str) {
      return str.indexOf('abe') > -1 && (0, _.getAttr)(str, 'key').indexOf('.') > -1;
    }

    /**
     * Test if a string contains string key from {{#each}} block statement
     * @param  {String}  str string to test
     * @return {Boolean} true = this is a block content
     */

  }, {
    key: 'isEachStatement',
    value: function isEachStatement(str) {
      return str.indexOf('#each') > -1 || str.indexOf('/each') > -1;
    }

    /**
     * Encode / Escape && add data-abe attributs
     * @param  {String} block
     * @return {String} escaped string
     */

  }, {
    key: 'encodeAbe',
    value: function encodeAbe(block) {
      var matchAbe = block.match(/>\s*\{\{abe .*\}\}/g);
      if (matchAbe) {
        for (var i = 0; i < matchAbe.length; i++) {
          block = block.replace(matchAbe[i], ' data-abe="' + (0, _.getAttr)(matchAbe[i], 'key').replace('.', '[0]-') + '" >');
        }
      }
      matchAbe = block.match(/( [A-Za-z0-9\-\_]+="*{{.*}})/g);
      if (matchAbe) {
        for (var i = 0; i < matchAbe.length; i++) {
          if (typeof matchAbe !== 'undefined' && matchAbe !== null) {
            block = block.replace(matchAbe[i], ' data-abe-attr="' + matchAbe[i].split('=')[0].trim() + '"' + ' data-abe="' + (0, _.getAttr)(matchAbe[i], 'key').replace('.', '[0]-') + '" ' + matchAbe[i]);
          }
        }
      }
      return escape(block);
    }

    /**
     * Add some stuff like style / script before closing </body> tag
     * @param  {String} text html page
     * @return {String} text + some sugar stuff added on the fly
     */

  }, {
    key: 'insertDebugtoolUtilities',
    value: function insertDebugtoolUtilities(text) {
      return text.replace(/<\/body>/, '<style>\n          body [data-abe]{ transition: box-shadow 600ms ease-in-out; box-shadow: 0; }\n          body .select-border{ border-color: #007CDE; box-shadow: 0 3px 13px #7CBAEF; }\n          body img.display-attr:before { content: attr(alt); }\n          body a.display-attr:before { content: attr(title); }\n          body .display-attr:before { position: absolute; display: block; z-index: 555; font-size: 10px; background-color: rgba(255, 255, 255, 0.75); padding: 2px 5px; color: #5D5D5D; }\n          .hidden-abe{ display: none!important; width: 0px !important; height: 0px!important; position: absolute; left: -10000px; top: -10000px; visibility: hidden;}\n        </style>\n      </body>');
    }
  }, {
    key: 'lorem',
    value: function lorem(type, v) {
      var lorem = '';
      if (type === 'text') {
        lorem = (0, _loremIpsum2.default)({
          units: 'sentences' // Generate words, sentences, or paragraphs.
          , sentenceLowerBound: 5,
          sentenceUpperBound: 10
        });
      } else if (type === 'link') {
        lorem = 'http://www.google.com';
      } else if (type === 'image' || type === 'file') {
        var width = (0, _.getAttr)(v, 'width');
        width = width !== '' ? width : 300;
        var height = (0, _.getAttr)(v, 'height');
        height = height !== '' ? height : 300;
        lorem = 'http://placehold.it/' + height + 'x' + width;
      } else if (type === 'textarea' || type === 'rich') {
        lorem = (0, _loremIpsum2.default)({
          units: 'paragraphs' // Generate words, sentences, or paragraphs.
          , paragraphLowerBound: 3,
          paragraphUpperBound: 7
        });
      }
      return lorem;
    }
  }, {
    key: 'form',
    get: function get() {
      return this._form;
    }
  }], [{
    key: 'escapeRegExp',
    value: function escapeRegExp(str) {
      var specials = [
      // order matters for these
      "-", "[", "]"
      // order doesn't matter for any of these
      , "/", "{", "}", "(", ")", "*", "+", "?", ".", "\\", "^", "$", "|"]

      // I choose to escape every character with '\'
      // even though only some strictly require it when inside of []
      ,
          regex = RegExp('[' + specials.join('\\') + ']', 'g');
      return str.replace(regex, "\\$&");
    }
  }, {
    key: 'addMetas',
    value: function addMetas(tpl, json, type) {
      var obj = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
      var user = arguments.length <= 4 || arguments[4] === undefined ? {} : arguments[4];
      var date = arguments.length <= 5 || arguments[5] === undefined ? null : arguments[5];

      var meta = _.config.meta.name;

      json[meta] = (0, _extend2.default)({}, json[meta]);
      var currentDate = date || new Date();
      var username = user.name || '';
      var abeUrl = type === 'publish' ? json[meta].link : _.fileAttr.add(json[meta].link, 'd' + currentDate.toISOString()) + '';

      if (typeof json[meta].date === 'undefined' || json[meta].date === null) {
        json[meta].date = currentDate;
        json[meta].author = username;
      }
      json[meta].latest = {
        date: currentDate,
        author: username,
        abeUrl: abeUrl
      };

      if (typeof json[meta][type] === 'undefined' || json[meta][type] === null) {
        json[meta][type] = JSON.parse(JSON.stringify(obj));
        json[meta][type].date = currentDate;
        json[meta][type].author = username;
        json[meta][type].abeUrl = abeUrl;
      }
      json[meta][type].latest = JSON.parse(JSON.stringify(obj));
      json[meta][type].latest.date = currentDate;
      json[meta][type].latest.author = username;
      json[meta][type].latest.abeUrl = abeUrl;
    }
  }, {
    key: 'getDataList',
    value: function getDataList(tplPath, text, jsonPage) {
      var p = new _es6Promise.Promise(function (resolve, reject) {
        var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
            match;

        var sourceAttr = _.config.source.name;
        if (typeof jsonPage[sourceAttr] === 'undefined' || jsonPage[sourceAttr] === null) {
          jsonPage[sourceAttr] = {};
        }

        var promises = [];
        while (match = listReg.exec(text)) {

          var pSource = new _es6Promise.Promise(function (resolveSource, rejectSource) {
            var source = (0, _.getAttr)(match[0], 'source');
            var key = (0, _.getAttr)(match[0], 'key');
            var type = _.Sql.getSourceType(source);
            switch (type) {
              case 'request':
                var data = _.Sql.getDataRequest(tplPath, match[0], jsonPage);
                jsonPage[sourceAttr][key] = data;

                if (typeof jsonPage[key] !== 'undefined' && jsonPage[key] !== null) {
                  var newJsonValue = [];
                  Array.prototype.forEach.call(jsonPage[key], function (oldValue) {
                    Array.prototype.forEach.call(jsonPage[sourceAttr][key], function (newValue) {
                      if (typeof oldValue[_.config.meta.name] !== 'undefined' && oldValue[_.config.meta.name] !== null && oldValue[_.config.meta.name].link === newValue[_.config.meta.name].link) {
                        newJsonValue.push(newValue);
                      }
                    });
                  });
                  jsonPage[key] = newJsonValue;
                }
                resolveSource();
                break;
              case 'value':
                var value = _.Sql.getDataSource(match[0]);

                if (value.indexOf('{') > -1 || value.indexOf('[') > -1) {
                  try {
                    value = JSON.parse(value);

                    jsonPage[sourceAttr][key] = value;
                  } catch (e) {
                    jsonPage[sourceAttr][key] = null;
                    console.log(_cliColor2.default.red('Error ' + value + '/is not a valid JSON'), '\n' + e);
                  }
                }
                resolveSource();
                break;
              case 'url':
                // @TODO
                (0, _ajaxRequest2.default)(source, function (err, res, body) {
                  if (typeof body === 'string') {
                    var parsedBody = JSON.parse(body);
                    if ((typeof parsedBody === 'undefined' ? 'undefined' : _typeof(parsedBody)) === 'object' && Object.prototype.toString.call(parsedBody) === '[object Array]') {
                      jsonPage[sourceAttr][key] = parsedBody;
                    } else if ((typeof parsedBody === 'undefined' ? 'undefined' : _typeof(parsedBody)) === 'object' && Object.prototype.toString.call(parsedBody) === '[object Object]') {
                      jsonPage[sourceAttr][key] = [parsedBody];
                    }
                  } else if ((typeof body === 'undefined' ? 'undefined' : _typeof(body)) === 'object' && Object.prototype.toString.call(body) === '[object Array]') {
                    jsonPage[sourceAttr][key] = body;
                  } else if ((typeof body === 'undefined' ? 'undefined' : _typeof(body)) === 'object' && Object.prototype.toString.call(body) === '[object Object]') {
                    jsonPage[sourceAttr][key] = body;
                  }
                  resolveSource();
                });

                break;
              case 'file':
                jsonPage[sourceAttr][key] = _.FileParser.getJson(_.fileUtils.concatPath(_.config.root, source));
                resolveSource();
                break;
              default:
                resolveSource();
                break;
            }
          });
          promises.push(pSource);
        }

        _es6Promise.Promise.all(promises).then(function () {
          resolve();
        }).catch(function (e) {
          console.error(e.stack);
        });
        // return filesRequest
      }).catch(function (e) {
        console.error(e.stack);
      });

      return p;
    }
  }, {
    key: 'removeDataList',
    value: function removeDataList(text) {
      var listReg = /({{abe.*type=[\'|\"]data.*}})/g;

      _.log.delAndWrite('Util-getList', text);
      return text.replace(listReg, '');
    }
  }]);

  return Utils;
}();

exports.default = Utils;