'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.editor = editor;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _es6Promise = require('es6-promise');

var _cli = require('../../cli');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = new _cli.Util();
var arrayBlock = [];
var text;
var json;
var fakeContent = false;
var tabIndex = 0;

function add(type, key, desc, maxLength, tab, value, source, display, reload, order) {
  var required = arguments.length <= 10 || arguments[10] === undefined ? false : arguments[10];

  var hasSource = false;

  if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object' && Object.prototype.toString.call(type) === '[object Object]') {
    var key = type.key.split('[')[0];
    var index = type.key.match(/[^\[]+?(?=\])/)[0];
    var prop = type.key.replace(/[^\.]+?\./, '');
    if (fakeContent) {
      value = util.lorem(type.type);
    }
    if (typeof json[key] !== 'undefined' && json[key] !== null && typeof json[key][index] !== 'undefined' && json[key][index] !== null && typeof json[key][index][prop] !== 'undefined' && json[key][index][prop] !== null) {
      value = json[key][index][prop];
    } else if (typeof value !== 'undefined' && value !== null && value !== '') {
      if (typeof json[key] === 'undefined' || json[key] === null) json[key] = [];
      if (typeof json[key][index] === 'undefined' || json[key][index] === null) json[key][index] = {};
      json[key][index][prop] = value;
    }

    util.add(type.type, type.key, type.desc, type.maxLength, type.tab, value, source, display, type.reload, parseInt(type.order), type.required);
  } else {
    if ((typeof value === 'undefined' || value === null || value === '') && fakeContent) {
      if (typeof source === 'undefined' || source === null) {
        value = util.lorem(type);
      } else {
        if ((typeof source === 'undefined' ? 'undefined' : _typeof(source)) === 'object') {
          value = [];
          var i = 0;
          Array.prototype.forEach.call(source, function (item) {
            if (typeof maxLength === 'undefined' || maxLength === null || maxLength === '' || i < maxLength) {
              value.push(item);
            }
            i++;
          });
          json[key] = value;
        } else {
          value = source;
        }
      }
      json[key] = value;
    }

    util.add(type, key, desc, maxLength, tab, value, source, display, reload, parseInt(order), required);
  }

  return value;
}

function addToForm(match) {
  var keyArray = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
  var i = arguments.length <= 2 || arguments[2] === undefined ? 0 : arguments[2];

  var v = '{{' + match + '}}' // the match
  ,
      key = (0, _cli.getAttr)(v, 'key'),
      type = (0, _cli.getAttr)(v, 'type'),
      desc = (0, _cli.getAttr)(v, 'desc'),
      maxLength = (0, _cli.getAttr)(v, 'max-length'),
      reload = (0, _cli.getAttr)(v, 'reload'),
      order = (0, _cli.getAttr)(v, 'order'),
      tab = (0, _cli.getAttr)(v, 'tab'),
      required = (0, _cli.getAttr)(v, 'required');

  if (typeof keyArray !== 'undefined' && keyArray !== null) {
    var realKey = key.replace(/[^\.]+?\./, '');

    if (key.indexOf(keyArray + '.') >= 0 && realKey.length > 0) {
      insertAbeEach(keyArray, realKey, type, desc, maxLength, i, tab, null, reload, order, required);
    } else if (util.dontHaveKey(key)) {
      json[key] = add(type, key, desc, maxLength, tab, json[key], null, null, reload, order, required);
    }
  } else if (util.dontHaveKey(key) && util.isSingleAbe(v, text)) {
    var realKey = key.replace(/\./g, '-');
    json[realKey] = add(type, key, desc, maxLength, tab, json[realKey], null, null, reload, order, required);
  }
}

function matchAttrAbe() {
  var patt = /abe [^{{}}]+?(?=\}})/g,
      match;
  // While regexp match HandlebarsJS template item => keepgoing
  while (match = patt.exec(text)) {
    addToForm(match[0]);
  }
}

function insertAbeEach(keyArray, realKey, type, desc, maxLength, i, tab, source, reload, order, required) {
  if (typeof arrayBlock[keyArray][realKey] === "undefined" || arrayBlock[keyArray][realKey] === null) {
    arrayBlock[keyArray][realKey] = [];
  }
  var key = keyArray + "[" + i + "]." + realKey;
  var exist = false;
  Array.prototype.forEach.call(arrayBlock[keyArray][realKey], function (block) {
    if (block.key === key) {
      exist = true;
    }
  });
  if (!exist) {
    arrayBlock[keyArray][realKey].push({
      type: type,
      key: keyArray + "[" + i + "]." + realKey,
      desc: desc + " " + i,
      maxLength: maxLength,
      reload: reload,
      placeholder: '',
      tab: tab,
      order: order,
      required: required
    });
  }
}

function each() {
  var pattEach = /(\{\{#each (\r|\t|\n|.)*?\/each\}\})/g;
  var patt = /abe [^{{}}]+?(?=\}})/g;
  var textEach, match;

  while (textEach = pattEach.exec(text)) {
    var keyArray = textEach[0].match(/#each (\n|.)*?\}/);
    keyArray = keyArray[0].slice(6, keyArray[0].length - 1);

    if (keyArray.split(' ').length > 1) keyArray = keyArray.split(' ')[0];
    arrayBlock[keyArray] = [];
    // ce while boucle sur les block de contenu {{abe}}
    while (match = patt.exec(textEach[0])) {
      var v = match[0];

      if (v.indexOf('abe') > -1) {
        if (json[keyArray]) {
          for (var i = 0; i < json[keyArray].length; i++) {
            var key = json[keyArray];
            addToForm(v, keyArray, i);
          }
        } else {
          addToForm(v, keyArray, 0);
        }
      }
    }

    // ici on boucle a nouveau sur les champs pour les placer a la suite dans le formulaire
    var attrArray = [],
        length = 0;
    for (var index in arrayBlock[keyArray]) {
      attrArray.push(index);
      length = arrayBlock[keyArray][index].length;
    }

    for (var i = 0; i < length; i++) {
      for (var j = 0; j < attrArray.length; j++) {
        add(arrayBlock[keyArray][attrArray[j]][i]);
      }
    }
  }
}

function addSource() {
  var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
      match,
      limit = 0;

  while (match = listReg.exec(text)) {
    var type = 'text',
        editable = (0, _cli.getAttr)(match[0], 'editable'),
        key = (0, _cli.getAttr)(match[0], 'key'),
        display = (0, _cli.getAttr)(match[0], 'display'),
        desc = (0, _cli.getAttr)(match[0], 'desc'),
        maxLength = (0, _cli.getAttr)(match[0], 'max-length'),
        reload = (0, _cli.getAttr)(match[0], 'reload'),
        tab = (0, _cli.getAttr)(match[0], 'tab'),
        order = (0, _cli.getAttr)(match[0], 'order'),
        required = (0, _cli.getAttr)(match[0], 'required');

    editable = typeof editable === 'undefined' || editable === null || editable === '' || editable === 'false' ? false : true;

    var source = json[_cli.config.source.name][key];
    var eachSource = new RegExp('{{#each ' + key + '}');
    // maxLength = (eachSource.test(text)) ? maxLength : 1

    if (editable) {
      add(type, key, desc, maxLength, tab, json[key], source, display, reload, order, required);
    } else {
      json[key] = source;
    }
  }
}

function orderByTabindex(a, b) {
  if (a.order < b.order) {
    return -1;
  } else if (a.order > b.order) {
    return 1;
  }

  return 0;
}

function orderBlock() {

  var formBlock = {};

  for (var tab in util.form) {
    var formBlockTab = {};
    for (var i = 0; i < util.form[tab].item.length; i++) {
      var blockName = util.form[tab].item[i].block === '' ? 'default_' + i : util.form[tab].item[i].block;
      if (util.form[tab].item[i].key.indexOf('[') > -1) {
        blockName = util.form[tab].item[i].key.split('[')[0];
      }
      if (typeof formBlockTab[blockName] === 'undefined' || formBlockTab[blockName] === null) {
        formBlockTab[blockName] = [];
      }
      formBlockTab[blockName].push(util.form[tab].item[i]);
    }
    if (typeof blockName !== 'undefined' && blockName !== null) {
      formBlockTab[blockName].sort(orderByTabindex);
    }
    if (typeof formBlock[tab] === 'undefined' || formBlock[tab] === null) {
      formBlock[tab] = {};
    }

    var formBlockOrdered = {};
    var arKeys = Object.keys(formBlockTab).sort(function (a, b) {
      if (formBlockTab[a][0].order < formBlockTab[b][0].order) {
        return -1;
      } else if (formBlockTab[a][0].order > formBlockTab[b][0].order) {
        return 1;
      }
      return 0;
    });

    Array.prototype.forEach.call(arKeys, function (arKey) {
      formBlockOrdered[arKey] = formBlockTab[arKey];
    });
    formBlock[tab] = formBlockOrdered;
  }

  var formTabsOrdered = {};
  var arKeysTabs = Object.keys(formBlock).sort(function (a, b) {
    if (formBlock[a][Object.keys(formBlock[a])[0]][0].order < formBlock[b][Object.keys(formBlock[b])[0]][0].order) {
      return -1;
    } else if (formBlock[a][Object.keys(formBlock[a])[0]][0].order > formBlock[b][Object.keys(formBlock[b])[0]][0].order) {
      return 1;
    }
    return 0;
  });

  Array.prototype.forEach.call(arKeysTabs, function (arKeysTab) {
    formTabsOrdered[arKeysTab] = formBlock[arKeysTab];
  });

  return formTabsOrdered;
}

function editor(fileName, tplUrl, fake) {
  var p = new _es6Promise.Promise(function (resolve, reject) {
    tabIndex = 0;
    fakeContent = fake;
    util = new _cli.Util();

    json = {};
    if (_cli.fileUtils.isFile(tplUrl.json.path)) {
      json = _cli.FileParser.getJson(tplUrl.json.path, 'utf8');
    }

    text = (0, _cli.getTemplate)(fileName);

    _cli.Util.getDataList(_cli.fileUtils.removeLast(tplUrl.publish.link), text, json).then(function () {
      addSource();

      text = _cli.Util.removeDataList(text);

      matchAttrAbe(text, json);
      arrayBlock = [];
      each(text, json);

      var blocks = orderBlock();

      _cli.abeEngine.instance.content = json;

      resolve({
        form: blocks,
        json: json
      });
    }).catch(function (e) {
      console.error(e.stack);
    });
  });

  return p;
}