import Handlebars from 'handlebars'
import HandlebarsIntl from 'handlebars-intl'
import fse from 'fs-extra'
let singleton = Symbol()
let singletonEnforcer = Symbol()

import {
  cli
  ,log
  ,Sql
  ,config
  ,Locales
} from '../'

class abeEngine {

  constructor(enforcer) {
    if(enforcer != singletonEnforcer) throw "Cannot construct Json singleton"
    this._content = {}
  }

  static get instance() {
    if(!this[singleton]) {
      this[singleton] = new abeEngine(singletonEnforcer)
    }
    return this[singleton]
  }

  get content(){
    return this._content
  }

  set content(content){
    this._content = content
  }
}
export {abeEngine as abeEngine}

/**
 * Abe handlebar helper, that retrieve text to add to handlebars templating engine
 * @return {String} the string to replace {{ handlebars_key }}
 */
export function compileAbe(){
  var content = abeEngine.instance.content
  var keyAttr = (typeof arguments[0].hash['key_text'] !== 'undefined') ? 'key_text' : 'key'

  if(arguments[0].hash[keyAttr].indexOf(']-') > 0){

    var key = arguments[0].hash[keyAttr].split('-')
    key = key[key.length - 1]
    var hash = arguments[0].hash
    return new Handlebars.SafeString((content) ? content[hash['dictionnary']][arguments[0].data.index][key] : hash.key)
  }

  var key = arguments[0].hash[keyAttr].replace('.', '-')

  var hash = arguments[0].hash
  var value = (content) ? content[hash.key.replace('.', '-')] : hash.key
  if(typeof value === 'undefined' || value === null) {
    value = ''
  }
  return new Handlebars.SafeString(value)
}

/**
 * Print properties inside html tag
 * @param  {String} attr exemple: atl, title, ...
 * @param  {[type]} value value of the property
 * @return {String} the value to print inside the attribut
 */
export function attrAbe (attr, value) {
  var content = abeengine._content
  return new Handlebars.SafeString(value)
}

export function testObj (ctx) {
  console.log(ctx)
  console.log('\n\n----------\n\n')
}

export function ifIn (actions, currentAction, options) {
  for(var action in actions){
    if(action === currentAction) return options.fn(this)
  }
  return ''
}

function sourceOption(val, params) {
  var hiddenVal = val
  var selected = ''

  if(typeof hiddenVal === 'object' && Object.prototype.toString.call(hiddenVal) === '[object Object]') {
    hiddenVal = JSON.stringify(hiddenVal).replace(/'/g, "&apos;")

    var displayVal = Sql.deep_value_array(val, params.display)
    if(typeof params.display !== 'undefined' && params.display !== null
      && typeof displayVal !== 'undefined' && displayVal !== null) {
      val = displayVal
    }else {
      val = val[Object.keys(val)[0]]
    }
  }

  if(typeof params.value === 'object' && Object.prototype.toString.call(params.value) === '[object Array]') {
    Array.prototype.forEach.call(params.value, (v) => {
      var item = v
      var displayV = Sql.deep_value_array(item, params.display)
      if(typeof params.display !== 'undefined' && params.display !== null
        && typeof displayV !== 'undefined' && displayV !== null) {
        item = displayV
      }else {
        if(typeof v === 'string') {
          item = v
        }else {
          item = v[Object.keys(v)[0]]
        }
      }
      if(typeof val === 'object' && Object.prototype.toString.call(val) === '[object Array]'
        && typeof item === 'object' && Object.prototype.toString.call(item) === '[object Array]') {
        
        Array.prototype.forEach.call(item, (i) => {
          if(val.includes(i)) {
            selected = 'selected="selected"'
          }
        })
      }else if(val === item) {
        selected = 'selected="selected"'
      }
    })
  }else if(params.value === hiddenVal) {
    selected = 'selected="selected"'
  }
  return `<option value='${hiddenVal}' ${selected}>${val}</option>`
}

export function printBlock (ctx) {
  var res = ''

  if(typeof ctx[0].block !== 'undefined' && ctx[0].block !== null && ctx[0].block !== '') {
    res += `<div class="form-group">
              <label class="title">${ctx[0].block}</label>
              <div class='single-block well well-sm'>`
    Array.prototype.forEach.call(ctx, (item) => {
      res += printInput(item)
    })
    res += `</div></div>`
  }else if(ctx[0].key.indexOf('[') > -1) {
    var ctxBlock = ctx[0].key.split('[')[0]
    res += `<div class="form-group">
              <div class="list-group" data-block="${ctxBlock}" >
                <label>
                  ${ctxBlock}
                </label>`
    
    var arrItem = []
    Array.prototype.forEach.call(ctx, (item) => {
      var index = item.key.match(/[^\[\]]+?(?=\])/)
      if(typeof arrItem[index] === 'undefined' || arrItem[index] === null) {
        arrItem[index] = []
      }
      arrItem[index].push(item)
    })

    Array.prototype.forEach.call(Object.keys(arrItem), (i) => {
      var key = arrItem[i][0].key.split('[')[0]
      var display = ''
      if(typeof abeEngine.instance.content[key] === 'undefined' || abeEngine.instance.content[key] === null
        || abeEngine.instance.content[key].length === 0) {
        display = 'style="display: none"'
      }
      res += `<div class="list-block" data-block="${key}${i}" ${display}>
                              <button type="button" class="btn btn-info collapsed" data-toggle="collapse" data-target="#${key}${i}" >
                                <span class="glyphicon glyphicon-chevron-down" aria-hidden="true"></span>
                              </button>
                              <div id="${key}${i}" class="collapse" >`
      Array.prototype.forEach.call(arrItem[i], (item) => {
        res += printInput(item)
      })
      res += `</div></div>`
    })

    res +=  `
          </div>
          <button type="button" class="btn btn-success add-block" >
            <span class="glyphicon glyphicon-plus" aria-hidden="true"></span>
          </button><button type="button" class="btn btn-danger remove-block" >
            <span class="glyphicon glyphicon-minus" aria-hidden="true"></span>
          </button>
        </div>`
  }else {
    res += printInput(ctx[0])
  }
  return res
}

/**
 * Print form input based on input data type {Textarea | text | meta | link | image | ...}
 * && add appropriate attributs / data-attributs
 * @return {String|html} input / input group ...
 */
export function printInput () {
  var params = arguments[0],
      res = `<div class="form-group">
              <label for="${params.key}" 
                      ${(params.type.indexOf('text_link') > -1) ? 'data-for-link="' + params.key + '"' : ''} >
                ${params.description}
              </label>`,
      disabled = ''

  if(typeof params.placeholder === 'undefined' || params.placeholder === null || params.placeholder === 'undefined') {
    params.placeholder = ''
  }

  if(typeof params.value === 'undefined' || params.value === null) {
    params.value = ''
  }

  if(typeof params.source !== 'undefined' && params.source !== null) {
    var multiple = ''
    var disabled = ''
    if(typeof params.maxLength === 'undefined' || params.maxLength === null || params.maxLength === ''
      || (params.maxLength > 1 && params.source.length > 0)) {
      multiple = 'multiple'
    }
    if(params.source.length <= 0) {
      disabled = 'disabled'
    }
    var lastValues = JSON.stringify(params.value)

    res += `<select ${multiple} ${disabled} class="form-control form-abe"
                      id="${params.key}"
                      data-id="${params.key}"
                      value=''
                      last-values='${lastValues}'
                      maxlength="${params.maxLength}"
                      reload="${params.reload}"
                      tabIndex="${params.order}"
                      data-required="${params.required}"
                      placeholder="${params.placeholder}" >
            <option value=''></option>`

    if(typeof params.source === 'object' && Object.prototype.toString.call(params.source) === '[object Array]') {
      Array.prototype.forEach.call(params.source, (val) => {
        res += sourceOption(val, params)
      })

    }else {
      res += sourceOption(params.source, params)
    }

    res += `</select>`
  }else if (params.type.indexOf("rich") >= 0){
    res += `<div class="wysiwyg-container rich">
              <div class="wysiwyg-toolbar wysiwyg-toolbar-top">
                <a class="wysiwyg-toolbar-icon" href="#" title="Bold (Ctrl+B)" hotkey="b" data-action="bold" data-param="">
                  <span class="glyphicon glyphicon-bold"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Italic (Ctrl+I)" hotkey="i" data-action="italic" data-param="">
                  <span class="glyphicon glyphicon-italic"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Underline (Ctrl+U)" hotkey="u" data-action="underline" data-param="">
                  <span class="glyphicon underline">U</span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Text color" data-action="forecolor" data-param="" data-popup="color">
                  <span class="glyphicon glyphicon-text-color"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Background color" data-action="highlight" data-param="" data-popup="color">
                  <span class="glyphicon glyphicon-text-background"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Left" data-action="align" data-param="left">
                  <span class="glyphicon glyphicon-object-align-left"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Center" data-action="align" data-param="center">
                  <span class="glyphicon glyphicon-object-align-vertical"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Right" data-action="align" data-param="right">
                  <span class="glyphicon glyphicon-object-align-right"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Justify" data-action="justify" data-param="justify">
                  <span class="glyphicon glyphicon-menu-hamburger"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Subscript" data-action="subscript" data-param="">
                  <span class="glyphicon glyphicon-subscript"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Superscript" data-action="superscript" data-param="">
                  <span class="glyphicon glyphicon-superscript"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Indent" data-action="indent" data-param="">
                  <span class="glyphicon glyphicon-triangle-right"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Outdent" data-action="indent" data-param="outdent">
                  <span class="glyphicon glyphicon-triangle-left"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Unordered list" data-action="insertList" data-param="">
                  <span class="glyphicon glyphicon-th-list"></span>
                </a>
                <a class="wysiwyg-toolbar-icon" href="#" title="Remove format" data-action="removeFormat" data-param="">
                  <span class="glyphicon glyphicon-remove"></span>
                </a>
              </div>
              <textarea class="form-control form-abe form-rich"
                        id="${params.key}"
                        data-id="${params.key}"
                        maxlength="${params.maxLength}"
                        rows="4"
                        reload="${params.reload}"
                        data-required="${params.required}"
                        placeholder="${params.placeholder}" >${params.value}</textarea>
            </div>`
  }
  else if (params.type.indexOf("file") >= 0){
    res += `<input class="form-control"
                   data-id="${params.key}"
                   value="${params.value}"
                   name="${params.key}"
                   reload="${params.reload}"
                   data-required="${params.required}"
                   tabindex="${params.order}"
                   type="file" />
            <span class="percent"></span>
            <input type="text"
                   class="form-control form-abe hidden"
                   id="${params.key}"
                   data-id="${params.key}"
                   value="${params.value}"
                   maxlength="${params.maxLength}"
                   placeholder="${params.placeholder}" />`
  }
  else if (params.type.indexOf("textarea") >= 0){
    res += `<textarea class="form-control form-abe"
                      id="${params.key}"
                      data-id="${params.key}"
                      reload="${params.reload}"
                      data-required="${params.required}"
                      value="${params.value}"
                      tabindex="${params.order}"
                      maxlength="${params.maxLength}"
                      rows="4"
                      placeholder="${params.placeholder}" >${params.value}</textarea>`
  }
  else if (params.type.indexOf("link") >= 0){
    res += `<div class="input-group">
            <div class="input-group-addon link">
              <span class="glyphicon glyphicon-link" aria-hidden="true"></span>
            </div>
            <input  type="text"
                    class="form-control form-abe"
                    id="${params.key}"
                    data-id="${params.key}"
                    value="${params.value}"
                    maxlength="${params.maxLength}"
                    reload="${params.reload}"
                    data-required="${params.required}"
                    tabindex="${params.order}"
                    placeholder="${params.placeholder}" />
          </div>`
  }
  else if (params.type.indexOf("image") >= 0){
    res += `<div class="input-group">
            <div class="input-group-addon image">
              <span class="glyphicon glyphicon-picture" aria-hidden="true"></span>
            </div>
            <input  type="text"
                    class="form-control form-abe"
                    id="${params.key}"
                    data-id="${params.key}"
                    value="${params.value}"
                    maxlength="${params.maxLength}"
                    reload="${params.reload}"
                    data-required="${params.required}"
                    tabindex="${params.order}"
                    placeholder="${params.placeholder}" />
          </div>`
  }
  else {
    res += `<div class="input-group">
            <div class="input-group-addon">
              <span class="glyphicon glyphicon-font" aria-hidden="true"></span>
              </div>
              <input  ${disabled}
                      type="text"
                      class="form-control form-abe"
                      id="${params.key}"
                      data-id="${params.key}"
                      value="${params.value}"
                      maxlength="${params.maxLength}"
                      tabindex="${params.order}"
                      reload="${params.reload}"
                      data-required="${params.required}"
                      placeholder="${params.placeholder}" />
            </div>`
  }

  res += '</div>'
  return res
}

/**
 * Handlebars helper, conditionnal modulo
 * @param  {Int} num number to test
 * @param  {Int} mod modulo number
 * @param  {String} block text block content inside of {{#moduloIf}} ... {{/moduloIf}}
 * @return {String|html} if true return the block compiled by handlebar inside our template, if not void
 */
export function moduloIf(num, mod, block) {
  if (parseInt((num) + 1) % parseInt(mod) === 0) {
    return block.fn(this)
  }
}

/**
 * Handlebars helper, to print className and escape it string
 */
export function className(str) {
  return str.replace(/\.| |\#/g, '_')
}

/**
 * Handlebars helper, to print json object
 */
export function printJson(obj) {
  return JSON.stringify(obj)
}

function recursiveFolder(obj, index = 1, dataShow = '') {
  var classHidden = ''
  if(index > 1) {
    classHidden = 'hidden'
  }

  var id = `level-${index}`
  if(typeof dataShow !== 'undefined' && dataShow !== null && dataShow !== '') {
    id += `-${dataShow}`
  }
  var parent = obj[0] ? obj[0].cleanPath.split('/')[0] : ''
  var res = `<div class="form-group level-${index} ${classHidden}" data-parent="${parent}" data-shown="${dataShow}">
    <label for="${id}" class="col-sm-5 control-label">Level ${index}</label>
    <div class="col-sm-7">
      <select data-show-hide-sub-folder="true" id="${id}" class="form-control">
        <option data-level-hide="${index+1}"></option>`

  var sub = ''
      
  Array.prototype.forEach.call(obj, (o) => {
    res += `<option data-level-hide="${index+2}" data-level-show="${index+1}" data-show="${o.name.replace(/\.| |\#/g, '_')}">${o.name}</option>`

    if(typeof o.folders !== 'undefined' && o.folders !== null && o.folders.length > 0) {
      sub += recursiveFolder(o.folders, index+1, o.name.replace(/\.| |\#/g, '_'))
    }
  })

  res += `</select>
    </div>
  </div>`

  res += sub

  return res
}

export function folders(obj) {
  var res = recursiveFolder(obj)

  return res
}

function recursivePrintConfig(obj, key = '') {
  var res = ''

  if(typeof obj === 'object' && Object.prototype.toString.call(obj) === '[object Object]') {
    Array.prototype.forEach.call(Object.keys(obj), (k) => {
      var strKey = key
      if(strKey !== '') {
        strKey += '.'
      }
      strKey += k
      res += recursivePrintConfig(obj[k], strKey)
    })
  }else {
    res += `<div class="form-group">
      <label class="col-sm-4 control-label" for="${key}">${key}</label>
      <div class="col-sm-8">
        <input type="text" class="form-control" id="${key}" data-json-key="${key}" placeholder="${obj}" value="${obj}">
      </div>
    </div>`
  }

  return res
}

export function printConfig(obj) {
  var res = recursivePrintConfig(obj)

  return res
}

export function cleanTab(obj) {
  obj = replaceUnwantedChar(obj.replace(/ |&/g, '_'))

  return obj
}

export function math(lvalue, operator, rvalue, options) {
    lvalue = parseFloat(lvalue)
    rvalue = parseFloat(rvalue)
        
    return {
        "+": lvalue + rvalue,
        "-": lvalue - rvalue,
        "*": lvalue * rvalue,
        "/": lvalue / rvalue,
        "%": lvalue % rvalue
    }[operator]
}

/**
 */
export function notEmpty(variable) {
  if (typeof variable !== 'undefined' && variable !== null && variable !== '') {
    return block.fn(this)
  }else {
    return ''
  }
}

function replaceUnwantedChar(str) {
  var chars = {"’": '', "'": '', '\"': '', 'Š': 'S', 'š': 's', 'Ž': 'Z', 'ž': 'z', 'À': 'A', 'Á': 'A', 'Â': 'A', 'Ã': 'A', 'Ä': 'A', 'Å': 'A', 'Æ': 'A', 'Ç': 'C', 'È': 'E', 'É': 'E', 'Ê': 'E', 'Ë': 'E', 'Ì': 'I', 'Í': 'I', 'Î': 'I', 'Ï': 'I', 'Ñ': 'N', 'Ò': 'O', 'Ó': 'O', 'Ô': 'O', 'Õ': 'O', 'Ö': 'O', 'Ø': 'O', 'Ù': 'U', 'Ú': 'U', 'Û': 'U', 'Ü': 'U', 'Ý': 'Y', 'Þ': 'B', 'ß': 'Ss', 'à': 'a', 'á': 'a', 'â': 'a', 'ã': 'a', 'ä': 'a', 'å': 'a', 'æ': 'a', 'ç': 'c', 'è': 'e', 'é': 'e', 'ê': 'e', 'ë': 'e', 'œ': 'oe', 'ì': 'i', 'í': 'i', 'î': 'i', 'ï': 'i', 'ð': 'o', 'ñ': 'n', 'ò': 'o', 'ó': 'o', 'ô': 'o', 'õ': 'o', 'ö': 'o', 'ø': 'o', 'ù': 'u', 'ú': 'u', 'û': 'u', 'ý': 'y', 'þ': 'b', 'ÿ': 'y'}
  for(var prop in chars) str = str.replace(new RegExp(prop, 'g'), chars[prop])
  return str
}

export function translate(lang, str) {
  var trad = Locales.instance.i18n
  if(typeof trad[lang] !== 'undefined' && trad[lang] !== null
    && typeof trad[lang][str] !== 'undefined' && trad[lang][str] !== null) {
    return trad[lang][str]
  }
  return str
}

Handlebars.registerHelper('abe', compileAbe) // HandlebarsJS unique text helper
Handlebars.registerHelper('i18nAbe', translate) // HandlebarsJS unique text helper
Handlebars.registerHelper('math', math) // HandlebarsJS unique text helper
Handlebars.registerHelper('moduloIf', moduloIf) // HandlebarsJS helper for modulo test
Handlebars.registerHelper('testObj', testObj) // 
Handlebars.registerHelper('cleanTab', cleanTab) // 
Handlebars.registerHelper('attrAbe', attrAbe) // 
Handlebars.registerHelper('notEmpty', notEmpty) 
Handlebars.registerHelper('printJson', printJson) 
Handlebars.registerHelper('className', className) 
Handlebars.registerHelper('folders', folders)
Handlebars.registerHelper('printConfig', printConfig)
Handlebars.registerHelper('ifIn', ifIn)
Handlebars.registerHelper('printInput', printInput)
Handlebars.registerHelper('printBlock', printBlock)
Handlebars.registerHelper('abeImport', abeImport)
HandlebarsIntl.registerWith(Handlebars)

export function abeImport (file, config, ctx) {
  var config = JSON.parse(config)
  var defaultPartials = `${__dirname.replace(/\/$/,"")}/${config.defaultPartials.replace(/\/$/,"")}`
  var partials = (config.partials !== '') ? `${config.root.replace(/\/$/,"")}/${config.partials.replace(/\/$/,"")}` : defaultPartials
  var pathToPartial = `${partials}/${file}.html`
  try{
    var stat = fse.statSync(pathToPartial)
  }
  catch(e){
    var pathToPartial = `${defaultPartials}/${file}.html`
  }
  var html = fse.readFileSync(pathToPartial, 'utf8')
  var template = Handlebars.compile(html)
  return new Handlebars.SafeString(template(ctx))
}
