import fse from 'fs-extra'
import extend from 'extend'
import {Promise} from 'es6-promise'

import {
  getAttr,
  getEnclosingTags,
  Util,
  config,
  fileUtils,
  abeEngine,
  fileAttr,
  cli,
  log,
  getTemplate,
  FileParser
} from '../../cli'

var util = new Util()
var arrayBlock = []
var text
var json
var fakeContent = false
var tabIndex = 0

function add(type, key, desc, maxLength, tab, value, source, display, reload, order, required = false) {
  var hasSource = false
  
  if(typeof type === 'object' && Object.prototype.toString.call(type) === '[object Object]') {
    var key = type.key.split('[')[0]
    var index = type.key.match(/[^\[]+?(?=\])/)[0]
    var prop = type.key.replace(/[^\.]+?\./, '')
    if(fakeContent) {
      value = util.lorem(type.type)
    }
    if(typeof json[key] !== 'undefined' && json[key] !== null &&
       typeof json[key][index] !== 'undefined' && json[key][index] !== null &&
       typeof json[key][index][prop] !== 'undefined' && json[key][index][prop] !== null) {
      value = json[key][index][prop]
    }else if(typeof value !== 'undefined' && value !== null && value !== '') {
      if(typeof json[key] === 'undefined' || json[key] === null) json[key] = []
      if(typeof json[key][index] === 'undefined' || json[key][index] === null) json[key][index] = {}
      json[key][index][prop] = value
    }
    
    util.add(type.type, type.key, type.desc, type.maxLength, type.tab, value, source, display, type.reload, parseInt(type.order), type.required)
  }else {
    if((typeof value === 'undefined' || value === null || value === '') && fakeContent) {
      if(typeof source === 'undefined' || source === null) {
        value = util.lorem(type)
      }else {
        if (typeof source === 'object') {
          value = []
          var i = 0
          Array.prototype.forEach.call(source, (item) => {
            if(typeof maxLength === 'undefined' || maxLength === null || maxLength === '' || i < maxLength) {
              value.push(item)
            }
            i++
          })
          json[key] = value
        }else {
          value = source
        }
      }
      json[key] = value
    }

    util.add(type, key, desc, maxLength, tab, value, source, display, reload, parseInt(order), required)
  }

  return value
}

function addToForm(match, keyArray = null, i = 0) {
  var v = `{{${match}}}` // the match
    , key = getAttr(v, 'key')
    , type = getAttr(v, 'type')
    , desc = getAttr(v, 'desc')
    , maxLength = getAttr(v, 'max-length')
    , reload = getAttr(v, 'reload')
    , order = getAttr(v, 'order')
    , tab = getAttr(v, 'tab')
    , required = getAttr(v, 'required')

  if(typeof keyArray !== 'undefined' && keyArray !== null) {
    var realKey = key.replace(/[^\.]+?\./, '')

    if(key.indexOf(keyArray + '.') >= 0 && realKey.length > 0){
      insertAbeEach(keyArray, realKey, type, desc, maxLength, i, tab, null, reload, order, required)

    }else if(util.dontHaveKey(key)) {
      json[key] = add(type, key, desc, maxLength, tab, json[key], null, null, reload, order, required)
    }

  }else if(util.dontHaveKey(key) && util.isSingleAbe(v, text)) {
    var realKey = key.replace(/\./g, '-')
    json[realKey] = add(type, key, desc, maxLength, tab, json[realKey], null, null, reload, order, required)
  }
}

function matchAttrAbe() {
  var patt = /abe [^{{}}]+?(?=\}})/g,
      match
  // While regexp match HandlebarsJS template item => keepgoing
  while (match = patt.exec(text)) {
    addToForm(match[0])
  }
}

function insertAbeEach (keyArray, realKey, type, desc, maxLength, i, tab, source, reload, order, required) {
  if(typeof arrayBlock[keyArray][realKey] === "undefined" || arrayBlock[keyArray][realKey] === null) {
    arrayBlock[keyArray][realKey] = []
  }
  let key = keyArray + "[" + i + "]." + realKey
  var exist = false
  Array.prototype.forEach.call(arrayBlock[keyArray][realKey], (block) => {
    if(block.key === key) {
      exist = true
    }
  })
  if(!exist) {
    arrayBlock[keyArray][realKey].push({
      type: type,
      key: keyArray + "[" + i + "]." + realKey,
      desc: desc + " " + i,
      maxLength: maxLength,
      reload: reload,
      placeholder: '',
      tab: tab,
      order: order,
      required: required
    })
  }
}

function each() {
  let pattEach = /(\{\{#each (\r|\t|\n|.)*?\/each\}\})/g
  let patt = /abe [^{{}}]+?(?=\}})/g
  var textEach, match

  while (textEach = pattEach.exec(text)) {
    var keyArray = textEach[0].match(/#each (\n|.)*?\}/)
    keyArray = keyArray[0].slice(6, keyArray[0].length - 1)

    if(keyArray.split(' ').length > 1) keyArray = keyArray.split(' ')[0]
    arrayBlock[keyArray] = []
    // ce while boucle sur les block de contenu {{abe}}
    while (match = patt.exec(textEach[0])) {
      var v = match[0]

      if(v.indexOf('abe') > -1){
        if(json[keyArray]){
          for (var i = 0; i < json[keyArray].length; i++) {
            var key = json[keyArray]
            addToForm(v, keyArray, i)
          }
        }else{
          addToForm(v, keyArray, 0)
        }
      }
    }

    // ici on boucle a nouveau sur les champs pour les placer a la suite dans le formulaire
    var attrArray = [],
        length = 0
    for(var index in arrayBlock[keyArray]) {
      attrArray.push(index)
      length = arrayBlock[keyArray][index].length
    }

    for (var i = 0; i < length; i++) {
      for (var j = 0; j < attrArray.length; j++) {
        add(arrayBlock[keyArray][attrArray[j]][i])
      }
    }
  }
}

function addSource() {
  var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
      match,
      limit = 0

  while (match = listReg.exec(text)) {
    var type = 'text'
    , editable = getAttr(match[0], 'editable')
    , key = getAttr(match[0], 'key')
    , display = getAttr(match[0], 'display')
    , desc = getAttr(match[0], 'desc')
    , maxLength = getAttr(match[0], 'max-length')
    , reload = getAttr(match[0], 'reload')
    , tab = getAttr(match[0], 'tab')
    , order = getAttr(match[0], 'order')
    , required = getAttr(match[0], 'required')
    
    editable = (typeof editable === 'undefined' || editable === null || editable === '' || editable === 'false') ? false : true

    var source = json[config.source.name][key]
    var eachSource = new RegExp(`{{#each ${key}}`)
    // maxLength = (eachSource.test(text)) ? maxLength : 1

    if(editable) {
      add(type, key, desc, maxLength, tab, json[key], source, display, reload, order, required)
    }else {
      json[key] = source
    }
  }
}

function orderByTabindex(a, b) {
  if(a.order < b.order) {
    return -1
  }else if(a.order > b.order) {
    return 1
  }

  return 0
}

function orderBlock() {
    
  var formBlock = {}

  for(var tab in util.form) {
    var formBlockTab = {}
    for (var i = 0; i < util.form[tab].item.length; i++) {
      var blockName = (util.form[tab].item[i].block === '') ? 'default_' + i : util.form[tab].item[i].block
      if(util.form[tab].item[i].key.indexOf('[') > -1){
        blockName = util.form[tab].item[i].key.split('[')[0]
      }
      if(typeof formBlockTab[blockName] === 'undefined' || formBlockTab[blockName] === null) {
        formBlockTab[blockName] = []
      }
      formBlockTab[blockName].push(util.form[tab].item[i])
    }
    if(typeof blockName !== 'undefined' && blockName !== null) {
      formBlockTab[blockName].sort(orderByTabindex)
    }
    if(typeof formBlock[tab] === 'undefined' || formBlock[tab] === null) {
      formBlock[tab] = {}
    }

    var formBlockOrdered = {}
    var arKeys = Object.keys(formBlockTab).sort((a,b) => {
      if(formBlockTab[a][0].order < formBlockTab[b][0].order) {
        return -1
      }else if(formBlockTab[a][0].order > formBlockTab[b][0].order) {
        return 1
      }
      return 0
    })

    Array.prototype.forEach.call(arKeys, (arKey) => {
      formBlockOrdered[arKey] = formBlockTab[arKey]
    })
    formBlock[tab] = formBlockOrdered
  }

  var formTabsOrdered = {}
  var arKeysTabs = Object.keys(formBlock).sort((a,b) => {
    if(formBlock[a][Object.keys(formBlock[a])[0]][0].order < formBlock[b][Object.keys(formBlock[b])[0]][0].order) {
      return -1
    }else if(formBlock[a][Object.keys(formBlock[a])[0]][0].order > formBlock[b][Object.keys(formBlock[b])[0]][0].order) {
      return 1
    }
    return 0
  })

  Array.prototype.forEach.call(arKeysTabs, (arKeysTab) => {
    formTabsOrdered[arKeysTab] = formBlock[arKeysTab]
  })

  return formTabsOrdered
}

export function editor(fileName, tplUrl, fake) {
  var p = new Promise((resolve, reject) => {
    tabIndex = 0
    fakeContent = fake
    util = new Util()

    json = {}
    if(fileUtils.isFile(tplUrl.json.path)) {
      json = FileParser.getJson(tplUrl.json.path, 'utf8')
    }
    
    text = getTemplate(fileName)

    Util.getDataList(fileUtils.removeLast(tplUrl.publish.link), text, json)
      .then(() => {
        addSource()

        text = Util.removeDataList(text)

        matchAttrAbe(text, json)
        arrayBlock = []
        each(text, json)

        var blocks = orderBlock()

        abeEngine.instance.content = json

        resolve({
          form: blocks,
          json: json
        })
      }).catch(function(e) {
        console.error(e.stack)
      })
  })

  return p
}